/* ale_getline.c --- Read a line from a file, allocated dynamically
   Jim Blandy <jimb@gnu.ai.mit.edu> --- July 1994 */

#include <stdio.h>

#include "xmalloc.h"
#include "ale_getline.h"

/* Read a line from FILE.  
   If we are initially at EOF, return -1.
   If we have some text to read, set *ADDRESS to a malloc'd block of
   space holding the text, and return its length.  */
int
ale_getline (FILE *file, char **address)
{
  int c;

  /* The length of the string in buf, and the size allocated.  */
  size_t buf_len, buf_size = 5;

  /* Temporary space for holding the line.  */
  char *buf = (char *) xmalloc (buf_size);

  /* Read the line.  */
  buf_len = 0;
  while ((c = getc (file)) != EOF && c != '\n')
    {
      if (buf_len >= buf_size)
        {
          buf_size *= 2;
          buf = xrealloc (buf, buf_size);
        }
      buf[buf_len++] = c;
    }

  if (c == EOF && buf_len == 0)
    {
      free (buf);
      return EOF;
    }
  else
    {
      *address = buf;
      return buf_len;
    }
}
