;; gene-menus.el (part of ale).
;; Copyright (C) 1993, 1994  Karl Fogel <kfogel@darwin.life.uiuc.edu>
;;
;; This software is copyrighted under the terms of the GNU General
;; Public License (see terms below).  The copyright covers this file,
;; specifically, and the program of which it is a part.
;; 
;;      This program is free software; you can redistribute it and/or modify
;;      it under the terms of the GNU General Public License as published by
;;      the Free Software Foundation; either version 1, or (at your option)
;;      any later version.
;; 
;;      This program is distributed in the hope that it will be useful,
;;      but WITHOUT ANY WARRANTY; without even the implied warranty of
;;      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;      GNU General Public License for more details.
;; 
;;      You should have received a copy of the GNU General Public License
;;      along with this program; if not, write to the Free Software
;;      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;;; gene-menus.el: menus for gene-mode.

;; Copyright (C) 1994 Karl Fogel

;; Author: Karl Fogel <kfogel@cs.oberlin.edu>
;; Maintainer: Karl Fogel <kfogel@cs.oberlin.edu>
;; Created: May 3 1994 (Mom's birthday, by the way).
;; Version: 0.0 (prototype)
;; Keywords: genes, ribosomes, sequences, molecular biology, milkshakes

;; Enough with the credits already, get on to the good stuff:

;;; Code:
(require 'gene-mode-tools)


;; gene-menu-generic-disabled-symbol
;; Some menus need this for blank spaces.
;; (Ours not to reason why, ours but to serve and die.)
(put 'gene-menu-generic-disabled-symbol
     'menu-enable
     '((lambda ())))


;;; helper functions
(defun gene-menu-make-binding (cell)
  (cons (nth 0 cell)
        (cons (nth 0 cell) (nth 1 cell))))


(defun gene-menu-dummy-function ()
  "Temporary filler for the menus."
  (interactive)
  (ding)
  (gene-message "Not yet implemented.")
  (ding))


(defun gene-choose-pointer (event)
  "Browse a menu of available mouse pointers and maybe choose one."
  (interactive "e")
  (let* ((shapes (apropos-internal "x-pointer-.*" 
                                   (lambda (sym)
                                     (and (symbol-value sym)
                                          (not (eq sym 'x-pointer-shape))))))
         (f-height (/ (frame-height) 2))
         (pane-list
          (let (temp-pane-list
                (iter 0))
            (while shapes
              (let (lst
                    (count 0))
                (while (and (< count f-height) shapes)
                  (let ((sym (car shapes)))
                    (setq lst (cons
                               (cons
                                (substring (symbol-name sym) 10) sym)
                               lst))
                    (setq shapes (cdr shapes))
                    (setq count (1+ count))))
                (setq iter (1+ iter))
                (setq
                 temp-pane-list
                 (cons
                  (cons
                   (format "-*- Pointer Choices %d -*-" iter)
                   (nreverse lst))
                  temp-pane-list))))
            (nreverse temp-pane-list)))
         (menu (cons "" pane-list))
         (choice (x-popup-menu event menu)))

    (setq x-pointer-shape (symbol-value choice))
    ;; Apparently, above only takes effect if one then sets the mouse
    ;; color.  Sigh.
    (set-mouse-color (face-foreground 'default))))


(defun gene-menu-popup-in-sequence-context (event)
  "Pop up the right menu, based on the context around the mouse pointer."
  ;; Some thought should be given to these menus.  What contexts are
  ;; there, what ought to be on a menu and what should be primarily
  ;; keyboard operations?
  (interactive "e")
  (let* ((here (posn-point (event-start event)))
         (sym (gene-get-here-sym here gene-seq-buffer)))
    (cond
     ((get-text-property here 'gene-selection)
      ;; In a selection
      (gene-menu-popup-selection-menu here))
     (t
      ;; Else just pop up the generic sequence menu
      (gene-menu-popup-seq-menu sym))
     )))


(defun gene-menu-popup-in-id-context (event)
  "Pop up the right menu, based on the context around the mouse pointer."
  ;; Some thought should be given to these menus.  What contexts are
  ;; there, what ought to be on a menu and what should be primarily
  ;; keyboard operations?
  (interactive "e")
  (let* ((here (posn-point (event-start event)))
         (sym (gene-get-here-sym here gene-id-buffer))
         (sel-group (gene-sgroup-selected-group-at-seq sym)))
    (cond
     (sel-group
      ;; In a group
      (gene-menu-popup-sgroup-command-menu sel-group))
     (sym
      ;; Else just pop up the generic id menu
      (gene-menu-popup-seq-menu sym))
     (t
      nil)
     )))

(defun gene-menu-popup-in-sgroup-frame-context (event)
  "Pop up a group menu, based on the context around the mouse pointer."
  ;; Some thought should be given to these menus.  What contexts are
  ;; there, what ought to be on a menu and what should be primarily
  ;; keyboard operations?
  (interactive "e")
  (let* ((here (posn-point (event-start event)))
         (group (get-text-property here 'group gene-sgroup-frame-buffer)))
    (if group
        (gene-menu-popup-sgroup-command-menu group))))
  

;; Cut and Paste:

(defun gene-menu-build-clipboard-menu (title)
  "Build a menu of items in the clipboard buffer, entitled TITLE."
  (list
   title
   (append

    (list title)
    
    ;; The groups:
    (gene-cb-make-sgroup-menu)
    "" ; leave a blank line in the menu
    ;; And the sequences
    (gene-cb-make-sequence-menu))))


(defun gene-menu-popup-clipboard-menu (title)
  "Popup a menu TITLE showing the clipboard's contents, return user's choice.
The choice is a group or a sequence in the clipboard."
  (interactive)
  (let ((menu (gene-menu-build-clipboard-menu title)))
    (if (cdr (nth 1 menu))
        (x-popup-menu t menu))))


(defun gene-menu-popup-paste-menu (&optional sym)
  "Popup a menu of pastable clipboard entries, return one."
  (interactive)
  (let ((line (if sym 
                  (gene-seq-line-number sym)
                (save-excursion
                  (set-buffer gene-seq-buffer)
                  (gene-current-line))))
        (choice (gene-menu-popup-clipboard-menu "-*- Paste -*-")))
    (if choice
        (cond
         ((symbolp choice)
          (gene-cb-paste-sequence choice line))
         ((listp choice)
          (gene-cb-paste-sgroup choice line))
         (t ; do nothing
          nil)))
    ))
          


(defun gene-menu-popup-cb-item-remove-menu (&optional sym)
  "Popup a menu of removable clipboard entries, return one."
  (interactive)
  (let ((choice (gene-menu-popup-clipboard-menu "-*- Remove Item -*-")))
    (if choice
        (cond
         ((symbolp choice)
          (gene-cb-remove-sequence choice))
         ((listp choice)
          (gene-cb-remove-sgroup choice))
         (t ; do nothing
          nil)))))

  
(defun gene-menu-popup-cb-all-remove-menu (&optional sym)
  "Erase the clipboard, after making sure the user means it."
  (interactive)
  (if (gene-y-or-n-p
       "Erase entire contents of clipboard?")
      (progn
        (gene-cb-remove-object nil nil gene-cb-sequence-clipboard)
        (gene-cb-remove-object nil nil gene-cb-sgroup-clipboard))
    (gene-message "Clipboard not erased.")))


;;; Menu maps:

;; Movement menu:
(defconst gene-menu-movement-menu
  (make-sparse-keymap "-*- Movement -*-")
  "The Movement menu keymap.")


(nconc
 gene-menu-movement-menu
 (mapcar
  'gene-menu-make-binding
  '(
    (""                       gene-menu-generic-disabled-symbol)
    ("Up Screen"              gene-seq-scroll-buffer-down)
    ("Down Screen"            gene-seq-scroll-buffer-up)
    ("Left Screen"            gene-seq-scroll-buffer-right)
    ("Right Screen"           gene-seq-scroll-buffer-left)  
    (""                       gene-menu-generic-disabled-symbol)
    ("Beginning of Sequence"  gene-seq-first-col)
    ("End of Sequence"        gene-seq-last-col)
    ("Top of Alignment"       gene-seq-first-sequence)
    ("Bottom of Alignment"    gene-seq-last-sequence)
    (""                       gene-menu-generic-disabled-symbol)
    ("Go to Residue"          gene-seq-goto-base)
    ("Go to Column"           gene-goto-column-interactive)
    (""                       gene-menu-generic-disabled-symbol)
    ("Up Residue"             gene-seq-up-base)
    ("Down Residue"           gene-seq-down-base)
    ("Up to Diff. Residue"    gene-seq-up-to-different-base)
    ("Down to Diff. Residue"  gene-seq-down-to-different-base)
    )))


;; Search menu
(defconst gene-menu-search-menu
  (make-sparse-keymap "-*- Searches -*-")
  "The Search menu keymap.")


(nconc
 gene-menu-search-menu
 (mapcar
  'gene-menu-make-binding
  '(
    (""                              gene-menu-generic-disabled-symbol)
    ("Search Forward in Sequence"    gene-search-forward-in-sequence)
    ("Search Backward in Sequence"   gene-search-backward-in-sequence)
    ("Search Forward in Alignment"   gene-search-forward-in-alignment)
    ("Search Backward in Alignment"  gene-search-backward-in-alignment)
    (""                              gene-menu-generic-disabled-symbol)
    ("Repeat Above Search"           gene-search-repeat-last-search)
    (""                              gene-menu-generic-disabled-symbol)
    ("Search for Organism By Name"   gene-seq-goto-organism)
    (""                              gene-menu-generic-disabled-symbol)
    ("Unhighlight Search Matches"    gene-search-unhighlight-all-matches)
    (""                              gene-menu-generic-disabled-symbol)
    ("Annotation Find-All"           gene-ann-search-and-build-group)
    ("Short-ID Find-All"             gene-id-search-and-build-group)
    )))


;; File menu
(defun gene-menu-popup-file-menu ()
  "Pop up the File menu, execute the chosen item if any."
  (interactive)
  (let* ((menu (gene-menu-build-file-menu))
         (choice (x-popup-menu t menu)))
    (if choice
        (call-interactively choice))))


(defun gene-menu-build-file-menu ()
  "Build the File Menu, suitably for x-popup-menu."
  (list
   "-*- File -*-"
   (append
    '("-*- File -*-")
    '("")
    (if (gene-alignment-empty-p)
        '(("Open"    .  gene-open-sequence-file))
      '("Open"))
    '(("Insert"  .  gene-insert-sequence-file))
    '(("Open Phylogenetic Listing" .  gene-phylo-list))
    (if gene-modified-p
        '(("Save" . gene-save-sequences))
      '("Save"))
    '(("Save As" . gene-save-sequences-as))
    '("")
    (if gene-modified-p
        '(("Quit (offers to save)" . gene-quit-maybe-offer-save))
      '(("Quit" . gene-quit-maybe-offer-save)))
    )))


;; Edit menu
(defun gene-menu-popup-edit-menu ()
  (interactive)
  (let* ((menu (gene-menu-build-edit-menu))
         (choice (x-popup-menu t menu)))
    (if choice
        (call-interactively choice))))


(defun gene-menu-build-edit-menu ()
  "Build the Edit Menu, suitably for x-popup-menu."
  (list
   "-*- Edit -*-"
   (list
    "-*- Edit -*-"
    ""
    ;; All the standard editing commands go here
    '("Throw to Right   (C-M-r)" . gene-seq-throw-right)
    '("Throw to Left    (C-M-l)" . gene-seq-throw-left)
    '("Fetch from Right (M-r)" . gene-seq-fetch-right)
    '("Throw from Left  (M-l)" . gene-seq-fetch-left)
    '("Slide to Right   (C-r)" . gene-seq-slide-right)
    '("Slide to Left    (C-l)" . gene-seq-slide-left)
    ""
    '("Delete Forward   (DEL)" . gene-seq-delete-char)
    '("Delete Backward  (BS)"  . gene-seq-delete-backward-char)
    ""
    '("Insert Gap Here      (TAB)" . gene-seq-insert-context-gap)
    '("Insert Gap Elsewhere (M-TAB)" . gene-seq-insert-gap-column-elsewhere)
    '("Delete Gap Column" . gene-seq-delete-superfluous-gap-column)
    ""
    (if (gene-clipboard-empty-p)
        "Clear Clipboard Item"
      '("Clear Clipboard Item" . gene-menu-popup-cb-item-remove-menu))
    (if (gene-clipboard-empty-p)
        "Clear Whole Clipboard"
      '("Clear Whole Clipboard" . gene-menu-popup-cb-all-remove-menu))
    ""
    '("Undo             (M-u)" . gene-undo)
    ""
    '("Cancel All Selections" . gene-sel-cancel-all-selections)
    ""
    '("Create New Sequence" . gene-new-sequence)
    ""
    '("Empty Alignment"     . gene-seq-empty-alignment)
    )))


;; Views menu
(defun gene-menu-popup-views-menu ()
  (interactive)
  (let* ((menu (gene-menu-alignment-views))
         (choice (x-popup-menu t menu)))
    (if choice
        (funcall choice))))


(defun gene-menu-alignment-views ()
  "Build the alignment frame's Views Menu, suitably for x-popup-menu."
  (list
   "-*- Views -*-"
   (append
    '("-*- Views -*-")
    '("")
    '(("Refresh" . gene-arrange-windows))
    (list (cons (if gene-window-split-p 
		    "Unsplit Alignment"
		  "Split Alignment Vertically")
		'gene-split-unsplit-window-horizontally))

    '("")

    (if (and gene-sgroup-frame
             (frame-live-p gene-sgroup-frame)
             (frame-visible-p gene-sgroup-frame))
        '(("Hide Group List" . gene-sgroup-frame-hide-frame))
      '(("Show Group List" . gene-sgroup-frame-show-frame)))
    
    (if gene-posmon-monitoring-on
        '(("Hide Position Numbers" . gene-posmon-monitoring-off))
      '(("Show Position Numbers"  . gene-posmon-monitoring-on)))
    
    (if (get-buffer-window gene-status-buffer t)
        ;; Mm, what nice consistent function naming!
        '(("Hide Lock Status" . gene-hide-status-window))
      '(("Show Lock Status" . gene-display-status-window)))
    
    (if (and gene-ann-frame
             (frame-live-p gene-ann-frame)
             (frame-visible-p gene-ann-frame))
        '(("Hide Annotations" . gene-ann-hide-frame))
      '(("Show Annotations" . gene-ann-show-frame)))

    '("")
    
    (gene-phylo-common-views)
    )))
    

;; Settings Menu
(defun gene-menu-popup-settings-menu ()
  (interactive)
  (let* ((menu (gene-menu-build-settings-menu))
         (choice (x-popup-menu t menu)))
    (if choice
        (funcall choice))))


(defun gene-menu-build-settings-menu ()
  "Build the Settings Menu, suitably for x-popup-menu."
  (list
   "-*- Settings -*-"
   (append
    '("-*- Settings -*-")
    '("")

    (if gene-seq-overstrike-mode
        '(("Switch to Insert Mode" . gene-seq-toggle-overstrike-mode))
      '(("Switch to Overstrike Mode" . gene-seq-toggle-overstrike-mode)))

    (if gene-seq-vertical-scroll-bars
        '(("Turn off Scrollbars" . gene-seq-toggle-vertical-scroll-bars))
      '(("Turn on Scrollbars" . gene-seq-toggle-vertical-scroll-bars)))

    (if gene-data-input-mode
        '(("Turn off Data Entry Mode" . gene-data-input-mode))
      '(("Turn on Data Entry Mode" . gene-data-input-mode)))


    '("")
    '(("Reverse Display" . gene-reverse-color-set))
    '("")
    (gene-generate-color-set-menu)
    '("")
    (gene-frame-generate-font-menu))))



;; selection menu:

(defun gene-menu-popup-selection-menu (spot)
  "Pop up a menu of commands that operate on selections."
  (interactive)
  (let* ((selection (gene-sel-selection-at-pos spot))
         (func (x-popup-menu t gene-menu-selection-command-menu)))
    (if func
        (gene-save-excursion-in
            gene-seq-buffer
          (funcall func selection)))))


(defconst gene-menu-selection-command-menu  
  '("-*- Selections -*-"
    ;; This menu has only one pane
    ("-*- Selection Commands -*-"
     ""
     ("Left Justify" . gene-sel-left-justify)
     ("Right Justify" . gene-sel-right-justify)
     ("Center Justify" . gene-sel-center-justify)
     ""
     ("Throw Right" . gene-sel-right-throw-selection)
     ("Throw Left" . gene-sel-left-throw-selection)
     ""
     ("Substitute Char (prompts)" . gene-sel-subst-char-in-selection)
     ("Upper Case" . gene-sel-upcase-selection)
     ("Lower Case" . gene-sel-downcase-selection)
     ("Flip Case"  . gene-sel-flipcase-selection)
     ("Cycle Lock Status" . gene-sel-next-lock-status)
     ""
     ("Search in Selection" . gene-search-matches-in-selection)
     ""
     ("InterWindow Copy" . gene-sel-copy-selection-for-interprogram-paste)
     ("Save to File" . gene-sel-save-selection-to-file)
     ""
     ("Cancel Selection" . gene-sel-cancel-selection)
     ""
     ("Delete Selection's Indels" . gene-sel-delete-selection-indels)
     ""
     ("Delete Selection's Contents" . gene-sel-delete-selection-contents)
     ;; ""
     ;; "U N I M P L E M E N T E D"
     ;; ("Search Forward" . gene-search-forward-in-selection)
     ;; ("Search Backward" . gene-search-backward-selection)
     )))


;; sequence menu

(defun gene-menu-popup-seq-menu (sym)
  "Pop up a menu of commands that operate on a sequence.
If the sequence is a member of the selected group, then the commands
may operate on the whole group, where applicable."
  (interactive "d")
  (let* ((org-name (gene-organism-name sym))
         (menu (list 
                "" ; what's the point of this string, anyway?
                (append
                 (list
                  (concat "-*- " org-name " -*-"))
                 (gene-menu-build-seq-menu))))
         (command (x-popup-menu t menu)))
    (if command
        (gene-save-excursion-in gene-seq-buffer
          (funcall command sym)))
    (gene-handle-vertical-motion)
    ))


;; Not really a menu -- requires titles to be appended to front before
;; it will work.  However, the popup function above does this.
(defun gene-menu-build-seq-menu ()
  "Build the per-sequence-menu."
  (list
   ""
   ;; Grey out if no sequence to cut.
   (if (gene-get-here-sym)
       '("Cut" . gene-cb-cut-sequence)
     "Cut")
   (if (gene-get-here-sym)
       '("Copy" . gene-cb-copy-sequence)
     "Copy")
   (if (gene-clipboard-empty-p)
       "Paste"
     '("Paste" . gene-menu-popup-paste-menu))
   (if (gene-clipboard-empty-p)
       "Clear Clipboard Item"
     '("Clear Clipboard Item" . gene-menu-popup-cb-item-remove-menu))
   (if (gene-clipboard-empty-p)
       "Clear Whole Clipboard"
     '("Clear Whole Clipboard" . gene-menu-popup-cb-all-remove-menu))
   ""
   '("Save Sequence to File"    . gene-seq-save-sequence-as)
   ""
   '("Substitute Char (prompts)" . gene-seq-subst-char-in-sequence)
   '("Upper Case" . gene-seq-upcase-sequence)
   '("Lower Case" . gene-seq-downcase-sequence)
   '("Flip Case"  . gene-seq-flipcase-sequence)
   ""
   '("Reverse"                 . gene-trans-open-reverse-sequence)
   '("Complement"              . gene-trans-open-complement-sequence)
   '("Reverse Complement"      . gene-trans-open-reverse-complement-sequence)
   '("Translate to Amino Acid" . gene-translate-sequence-to-amino)
   ""
   '("Monitor Position"  . gene-posmon-add-sequence-interactive)
   ""
   '("Cycle Lock Status" . gene-locking-cycle-to-next-state)
   ""
   '("Delete Sequence"   . gene-seq-delete-sequence)
   ))


;; ID/group popup menu

(defun gene-menu-popup-sgroup-command-menu (sgroup)
  "Pop up a menu of commands that operate on a sequence group."
  (interactive (list (gene-sgroup-selected-here)))
  (let* ((group-name (gene-sgroup-get-name sgroup))
         (menu (list
                "" ; what's the point of this string, anyway?
                (append
                 (list
                  (concat "-*- (Group) \"" group-name "\" -*-"))
                 gene-menu-sgroup-command-menu)))
         (command (x-popup-menu t menu)))
    (if command
        (gene-save-excursion-in
            gene-seq-buffer
          (funcall command sgroup)))
    (gene-handle-vertical-motion)
    ))


(defconst gene-menu-sgroup-command-menu
  '(
    ""
    ("Cut Group" . gene-cb-cut-sgroup)
    ("Copy Group" . gene-cb-copy-sgroup)
    ""
    ("Hide Group" . gene-sgroup-hide-group)
    ("Display Only" . gene-sgroup-just-display-group)
    ("Cancel Group" . gene-sgroup-delete-group)
    ""
    ("Save  Group To File"        . gene-sgroup-save-group-to-file)
    ("Merge Group To File"        . gene-sgroup-merge-group-to-file)
    ("Write Group To Phylip File" . gene-sgroup-write-sgroup-to-phylip-file)
    ""
    ("Substitute Char (prompts)" . gene-sgroup-subst-char-in-group)
    ("Upper-Case Group" . gene-sgroup-upcase-group)
    ("Lower-Case Group" . gene-sgroup-downcase-group)
    ("Flip-Case  Group"  . gene-sgroup-flipcase-group)
    ""
    ("Consensus Sequence or Mask"  . gene-consensus-dialog-box)
    ""
    ("Cycle Group's Lock Status" . gene-locking-cycle-to-next-state)
    ;; ""
    ;; ("Search Forward  in Group" . gene-search-forward-in-sgroup)
    ;; ("Search Backward in Group" . gene-search-backward-in-sgroup)
    ))


;;;; Analysis menu

(defun gene-menu-popup-analysis-menu ()
  "Popup an analysis menu."
  (interactive)
  (let* ((str
          (if gene-sgroup-selected-group
              (format
               "Composition of group %s"
               (gene-sgroup-get-name gene-sgroup-selected-group))
            (format
             "Composition of sequence %s"
             (gene-organism-name (gene-get-here-sym)))))
         (menu (list
                "" ; what's the point of this string, anyway?
                (list
                 "-*- Analysis -*-"
                 ""
                 (cons str (lambda ()
                             (call-interactively 'genalyze-composition)))

                 '("Column Statistics" . gene-seq-display-column-stats)

                 )))
         (command (x-popup-menu t menu)))
    (if command
        (funcall command))))

       
;; threats menu:
;; Some perspective
(defconst gene-menu-threat-menu
  (make-sparse-keymap "-*- Threats -*-")
  "The Threats menu keymap.")


(defun gene-escape-to-emacs ()
  "Bring up a frame showing the standard \"*scratch*\" buffer."
  (interactive)
  (message "Escaping...")
  (save-excursion
    (set-buffer (get-buffer-create "*Escape Hatch*"))
    (lisp-interaction-mode)
    (gene-frame-show (current-buffer) '((minibuffer . t))))
  (message "Escaped to the Emacs jungle."))


(nconc
 gene-menu-threat-menu
 (mapcar
  'gene-menu-make-binding
  '(
    ("An Emacs Threat..."           gene-random-emacs-threat)
    ("An X Windows Threat..."       gene-random-x-threat)
    )))


;; help menu:
;; This gets combined with Emacs' native help, apparently.
;; Last gas for 100 miles
(defconst gene-menu-help-menu
  (make-sparse-keymap "-*- Help -*-")
  "The Help menu keymap.")


(nconc
 gene-menu-help-menu
 (mapcar
  'gene-menu-make-binding
  '(
    (""                         gene-menu-generic-disabled-symbol)
    ("Quick Reference"          gene-help-quick-reference)
    ("Online Manual"            gene-help-info)
    ("Email a Bug Report"       gene-report-bug)
    ("Show Version"             gene-show-version)
    ("Escape to Standard Emacs" gene-escape-to-emacs)
    
    ;; ("Tutorial"              gene-menu-dummy-function)
    ;; ("About"                 gene-menu-dummy-function)
    ;; ("Donations"             gene-menu-dummy-function)
    ;; ("Report a Bug"          gene-menu-dummy-function)
    ;; ("Comments/Suggestions?" gene-menu-dummy-function)
    )))


;; This does not go in gene-version.el because that file is generated
;; automatically in the make process and contains *only* the
;; defconst'd `gene-version'.
(defun gene-show-version ()
  "Display Ale's version number in the minibuffer."
  (interactive)
  (message "This is Ale version %s" gene-version))



(defun gene-menu-popup-annotation-search-menu (evt)
  (interactive "e")
  (let* ((menu (list
                "" ; what's the point of this string, anyway?
                (append
                 (list
                  "-*- Searches -*-")
                 gene-menu-ann-command-menu)))
         (command (x-popup-menu t menu)))
    (if command
        (call-interactively command))))


(defconst gene-menu-ann-command-menu
  '(
    ""
    ("Annotation Find-All" . gene-ann-search-and-build-group)
    ))


;; make sure that the menu bar & surroundings are the right thickness
(modify-frame-parameters (selected-frame) '((menu-bar-lines . 1)))

;; clear the menu bars:

(defvar gene-menu-shared-map (make-sparse-keymap "gene-menu-bar")
  "Menu bar keymap shared by sequence, ID, and other buffers.")


(defvar gene-dont-clear-menu-bar nil
  "Non-nil means don't clear the menu bar.
This is a temporary kludge to solve a problem of Karl's.
It will go away when we only clear the menu bars in the gene
buffers.  Ask Karl if you're curious about the problem.")


;; Emacs 19.29 has a menu-bug.  Until 19.30 comes out, we won't be
;; able to set functions as the bindings of menus in the menu-bar;
;; only keymaps will do.  The following functions are affected:
;;
;; gene-menu-popup-analysis-menu
;; gene-menu-popup-annotation-menu
;; gene-menu-popup-settings-menu
;; gene-menu-popup-views-menu
;; gene-menu-popup-edit-menu
;; gene-menu-popup-file-menu


;; Temporary way of getting everyone off the menus:
(if gene-dont-clear-menu-bar ; see doc string for explanation
    nil

  ;; Okay, everything is normal, go ahead and set up the menu bars.
  (define-key global-map [menu-bar] gene-menu-shared-map)
  
  ;; (define-key gene-id-mode-map [menu-bar] gene-menu-shared-map)
  ;; (define-key gene-seq-mode-map [menu-bar] gene-menu-shared-map)
  ;; (define-key gene-status-mode-map [menu-bar] gene-menu-shared-map)
  ;; Annotation menu should be different.  Should it actually be the
  ;; regular Emacs menus?  Hmm...
  (define-key gene-ann-mode-map [menu-bar] (make-sparse-keymap "menu-bar"))
  
  ;; Make the new menus appear
  
  ;; Because the ID, seq, and status buffers all share
  ;; `gene-menu-shared-map', the actual bindings need only be done in
  ;; one buffer.  The seq buffer is the appropriate choice:
  
  (define-key gene-seq-mode-map [menu-bar gene-threat]
    (cons "Threats" gene-menu-threat-menu))
  
  (define-key gene-seq-mode-map [menu-bar gene-help]
    (cons "Help" gene-menu-help-menu))
  
  (define-key gene-seq-mode-map [menu-bar gene-analyze]
    (cons "Analysis" 'gene-menu-popup-analysis-menu))
  
  (define-key gene-seq-mode-map [menu-bar gene-search]
    (cons "Searches" gene-menu-search-menu))
  
  (define-key gene-seq-mode-map [menu-bar gene-movement]
    (cons "Movement" gene-menu-movement-menu))
  
  ;; Annotation gets this one too, and it doesn't share a mapvar:
  (define-key gene-ann-mode-map [menu-bar gene-annotation-search]
    (cons "Search" 'gene-menu-popup-annotation-search-menu))
  
  ;; Settings
  (define-key gene-seq-mode-map [menu-bar gene-settings]
    (cons "Settings" 'gene-menu-popup-settings-menu))
  
  ;; views
  (define-key gene-seq-mode-map [menu-bar gene-views]
    (cons "Views" 'gene-menu-popup-views-menu))
  
  ;; edit 
  (define-key gene-seq-mode-map [menu-bar gene-edit]
    (cons "Edit" 'gene-menu-popup-edit-menu))
  
  ;; file
  (define-key gene-seq-mode-map [menu-bar gene-file]
    (cons "File" 'gene-menu-popup-file-menu))
  
  ;; Mouse-3 popup menu in seq buffer:
  (define-key gene-seq-mode-map [down-mouse-3]
    'gene-menu-popup-in-sequence-context)
  
  ;; Mouse-3 popup menu in ID buffer:
  (define-key gene-id-mode-map [down-mouse-3]
    'gene-menu-popup-in-id-context)
  
  ;; Mouse-3 popup menu in Group Frame:
  (define-key gene-sgroup-frame-mode-map [down-mouse-3]
    'gene-menu-popup-in-sgroup-frame-context)
  
  ;; binding the "keys"
  ;; (define-key gene-seq-mode-map [down-mouse-3] 'gene-menu-popup-some-menu)
  )


(provide 'gene-menus)
