;; gene.el (part of ale).
;; Copyright (C) 1993, 1994  Karl Fogel <kfogel@darwin.life.uiuc.edu>
;;
;; This software is copyrighted under the terms of the GNU General
;; Public License (see terms below).  The copyright covers this file,
;; specifically, and the program of which it is a part.
;; 
;;      This program is free software; you can redistribute it and/or modify
;;      it under the terms of the GNU General Public License as published by
;;      the Free Software Foundation; either version 1, or (at your option)
;;      any later version.
;; 
;;      This program is distributed in the hope that it will be useful,
;;      but WITHOUT ANY WARRANTY; without even the implied warranty of
;;      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;      GNU General Public License for more details.
;; 
;;      You should have received a copy of the GNU General Public License
;;      along with this program; if not, write to the Free Software
;;      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;;; gene-mode.el: Major mode(s) for editing gene sequences.

;; Copyright (C) 1993, 1994 Karl Fogel

;; Author: Karl Fogel <kfogel@cs.oberlin.edu>
;; Maintainer: Karl Fogel <kfogel@cs.oberlin.edu>
;; Created: Nov. 26 1993
;; Version: 0.0 (prototype)
;; Keywords: genes, ribosomes, sequences, molecular biology, milkshakes

;; LCD Archive Entry:
;; gene-mode|Karl Fogel|kfogel@cs.oberlin.edu|
;; Mode(s) for editing ribosomal RNA sequences.|
;; 26-Nov-93|Version: 0.0|~/modes/gene-mode.el.Z|

;; Enough with the credits already, get on to the good stuff:

;; FOR FURTHER INFORMATION:
;; anon ftp rdp.life.uiuc.edu:/pub/RDP/, or send mail to
;; server@rdp.life.uiuc.edu, with "help" as the subject.

;; FAVORITE CHINESE RESTAURANT:
;; Boy, that's a tough one.  Probably Hong Min, or maybe Emperor's
;; Choice (both in Chicago's Chinatown).  How about you?
;; The New Yen Jing in Champaign (4th and Green) is not bad, either,
;; even though it bills itself as "fast food".  Not on the order of
;; the other two, but good for lunch.

;;; Code:

(setq gc-cons-threshold (* 1000 1000))

;;; If the ALE_LISPDIR environment variable names a directory, add it
;;; to load-path.
(let ((dir (getenv "ALE_LISPDIR")))
  (if (and (stringp dir)
	   (file-directory-p dir))
      (setq load-path (cons dir load-path))))

;;; If the ALE_ARCHDIR environment variable names a directory, add it
;;; to exec-path.
(let ((dir (getenv "ALE_ARCHDIR")))
  (if (and (stringp dir)
	   (file-directory-p dir))
      (setq exec-path (cons dir exec-path))))

(require 'gene-message)       ; Keep this first...
(require 'gene-mode-tools)    ; ... and this second.

(require 'gene-consensus)
(require 'gene-dialog)
(require 'gene-display)
(require 'gene-error)
(require 'gene-frame)
(require 'gene-windows)
(require 'gene-locking)
(require 'gene-io)
(require 'gene-search)
(require 'gene-undo)
(require 'gene-seq-mode)
(require 'gene-selection)
(require 'gene-id-mode)
(require 'gene-sgroup)
(require 'gene-clipboard)
(require 'gene-ann-mode)
(require 'gene-help)
(require 'gene-threats)
(require 'gene-phylo)
(require 'gene-analyze)
(require 'gene-translate)
(require 'gene-posmon)

;; Generated automatically by the Ale Makefile.
(require 'gene-version)

;; This needs to be last.
(require 'gene-menus)


;;; Startup code.

(defun gene-startup ()
  "Do some sanity checks, and then some initialization.
`gene-seq-mode' has not been called yet.

Sanity checks:
  - Make sure we're running under X windows.
Initialization:
  - Load the global ale.el file.
  - Load the user's .ale file.
  - Figure out which fonts this X server has."
  
  ;; We require X-windows. 
  (if (null window-system)
      (let ((standard-output 'external-debugging-output))
	(princ "You need to be using a window system to run Ale.\r\n")
	(princ "Check the DISPLAY environment variable.\r\n")
	(kill-emacs 1)))
  
  ;; Create the distribution color sets.
  (gene-create-default-color-sets)
  
  ;; Load the global ale.el file (analagous to default.el).
  (condition-case nil
      (load "ale" t)
    (error
     (ding)
     (message "Error in global file ale.el.")))
  
  ;; Load the .ale file from the user's home directory (analogous to
  ;; .emacs).
  (condition-case nil
      (load (expand-file-name "~/.ale") t nil t)
    (error
     (ding)
     (message "Error in %s/.ale." (expand-file-name "~"))))
    
  ;; Figure out which fonts to use.
  (gene-frame-init-fonts)

  ;; We don't want the default GNU Emacs startup screen nor toolbar.
  (setq inhibit-startup-message t)
  (tool-bar-mode -1)
  )


(provide 'gene)
